PROVIDER_SETUP = [
    {"name": "OpenAI", "value": "openai", "provider_name": "openai", "openai_api_key": "", "default_model": "gpt-4o"},
    {"name": "Anthropic", "value": "anthropic", "provider_name": "anthropic", "anthropic_api_key": "", "default_model": "claude-3-5-sonnet"},
    {"name": "Gemini", "value": "gemini", "provider_name": "gemini", "gemini_api_key": "", "default_model": "gemini-2.0-flash-exp"},
    {"name": "Azure", "value": "azure", "provider_name": "azure", "azure_api_key": "", "azure_api_base": "", "azure_api_version": "", "default_model": "gpt-4o"},
    {"name": "vLLM Ollama", "value": "vllm_ollama", "provider_name": "hosted_vllm", "hosted_vllm_api_base": "", "default_model": "llama3.2:3b"}
]